/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is by Trevor Hobson
*
* The Initial Developer of the Original Code is
*   Trevor Hobson
* Portions created by the Initial Developer are Copyright (C) 2008
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

var objCleanupCall = {
	// DOM Serializer and prompt service. For debugging.
	nsIDOMSerializer: Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer),
	promptService: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),

	nsILocalFile: new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile"),
	preferences: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),
	prefPlatform: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.platform."),

// This should work here but in 64 bit world it does not so we are going to create when required.
//	nsISolRequest: Components.classes['@objection/solrequest;1'].getService().wrappedJSObject,

	debugLevel: 10,

	prefObserverRegister: function()
	{
		this._branch = this.preferences;
		this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this._branch.addObserver("", this, false);
	},

	prefObserverUnregister: function()
	{
		if(!this._branch)
		{
			return;
		}
		this._branch.removeObserver("", this);
	},

	observe: function(aSubject, aTopic, aData)
	{
//this.jsdump('OBJECTION aTopic: ' + aTopic + ' aData: ' +aData, 1);
		switch(aTopic)
		{
			case "profile-after-change":
				this.privateBrowsing('exit'); // The browser never opens in private browsing mode so exit Objection Private Mode (Should only be needed if the browser crashed)
				this.removeAutomatic('open');
				this.prefObserverRegister();
				break;

			case "private-browsing":
				this.privateBrowsing(aData);
				break;

			case "quit-application":
				this.removeAutomatic('close');
				this.prefObserverUnregister();
				break;

			case "nsPref:changed":
				if (aData === 'usePrivate')
				{
					this.privateBrowsing('pref');
				}
				break;

			default:
				throw Components.Exception("Unknown topic: " + aTopic);
		}
	},

	removeAutomatic: function(timeEvent)
	{
		this.getDebugLevel();

		var automaticRemove = 'never';
		try
		{
			automaticRemove = this.preferences.getCharPref("automaticRemove");
		}
		catch (err)	{}
		if (timeEvent === automaticRemove)
		{
			// Test all the platforms to see which one this is
			var flashAppDirPref = this.testFlashDir();

			// If there is a valid path delete files
			if (flashAppDirPref !== null)
			{
				// Init the File/Directory objects
				var flashAppDir = new this.nsILocalFile();
				var flashSettingDir = new this.nsILocalFile();
				var flashSettingFile = new this.nsILocalFile();
				var flashObjectDir = new this.nsILocalFile();

				flashAppDir.initWithPath(flashAppDirPref);
				var flashObjectDirSet = false;
				flashObjectDir.initWithFile(flashAppDir);
				flashObjectDir.appendRelativePath('#SharedObjects');
				if (flashObjectDir.exists() && flashObjectDir.isDirectory())
				{
					try
					{
						var entries = flashObjectDir.directoryEntries;
						while(entries.hasMoreElements())
						{
							var entry = entries.getNext();
							entry.QueryInterface(Components.interfaces.nsIFile);
							if (entry.isDirectory())
							{
								flashObjectDir.initWithFile(entry);
								flashObjectDirSet = true;
								break;
							}
						}
					} catch(err) {}
					// Delete site directories (including the files)
					if (flashObjectDirSet)
					{
						try
						{
							var entries = flashObjectDir.directoryEntries;
							while(entries.hasMoreElements())
							{
								var entry = entries.getNext();
								entry.QueryInterface(Components.interfaces.nsIFile);
								if (entry.isDirectory())
								{
									try
									{
										entry.remove(true);
										this.jsdump('OBJECTION Remove lso:success:' + entry.path, 2);
									}
									catch (err)
									{
										this.jsdump('OBJECTION Remove lso:failed:' + entry.path + ' err:' + err, 5);
									}
								}
							}
						} catch(err)	{}
					}
					else
					{
						this.jsdump('OBJECTION Could not find:flashObjectDir:' + flashObjectDir.path, 3);
					}
				}
				automaticRemove = false;
				try
				{
					automaticRemoveSite = this.preferences.getBoolPref("automaticRemoveSite");
				}
				catch (err) {}
				// Remove the site settings as well, if requested
				if (automaticRemoveSite)
				{
					// Locate settings directories
					flashSettingDir.initWithFile(flashAppDir);
					flashSettingDir.appendRelativePath('macromedia.com');
					flashSettingDir.appendRelativePath('support');
					flashSettingDir.appendRelativePath('flashplayer');
					flashSettingDir.appendRelativePath('sys');
					if (flashSettingDir.exists() && flashSettingDir.isDirectory())
					{
						this.jsdump('OBJECTION flashSettingDir:' + flashSettingDir.path, 3);
						// Delete all settings directories (including the files)
						try
						{
							var entries = flashSettingDir.directoryEntries;
							while(entries.hasMoreElements())
							{
								var entry = entries.getNext();
								entry.QueryInterface(Components.interfaces.nsIFile);
								if (entry.isDirectory())
								{
									try
									{
										entry.remove(true);
										this.jsdump('OBJECTION Remove setting:success:' + entry.path, 2);
									}
									catch (err)
									{
										this.jsdump('OBJECTION Remove setting:failed:' + entry.path + ' err:' + err, 5);
									}
								}
							}
						} catch(err) {}
						// Remove domains from the settings file
						flashSettingFile.initWithFile(flashSettingDir);
						flashSettingFile.append('settings.sol');
						if (flashSettingFile.exists() && flashSettingFile.isFile())
						{
//							var xmlSettingFile = this.nsISolRequest.solRead(flashSettingFile); // 64 bit fix
							var nsISolRequest = Components.classes['@objection/solrequest;1'].getService().wrappedJSObject; // 64 bit fix
							var xmlSettingFile = nsISolRequest.solRead(flashSettingFile); // 64 bit fix
							if (xmlSettingFile !== null)
							{
								var xpe = Components.classes["@mozilla.org/dom/xpath-evaluator;1"].createInstance(Components.interfaces.nsIDOMXPathEvaluator);
								var nsResolver = xpe.createNSResolver(xmlSettingFile);
								var numDomains = xpe.evaluate("//data/element[@name='domains']/@number", xmlSettingFile, null, 2, null);
								if (numDomains !== null && numDomains.resultType === 2)
								{
									var resultsDomains = xpe.evaluate("//data/element[@parent='" + numDomains.stringValue + "']", xmlSettingFile, nsResolver, 4, null);
									resultsDomains.QueryInterface(Components.interfaces.nsIDOMXPathResult);  // not necessary in FF3, only 2.x and possibly earlier
									var res;
									var listDomains = [];
									while (res = resultsDomains.iterateNext())
									{
										listDomains.push(res);
									}
									if (listDomains.length > 0)
									{
										for (var i=0; i < listDomains.length; i++)
										{
											xmlSettingFile.firstChild.removeChild(listDomains[i]);
										}
//										var writeSuccess = this.nsISolRequest.solWrite(xmlSettingFile, flashSettingFile); // 64 bit fix
										var writeSuccess = nsISolRequest.solWrite(xmlSettingFile, flashSettingFile); // 64 bit fix
										if (writeSuccess)
										{
											this.jsdump('OBJECTION Remove domains:success', 3);
										}
										else
										{
											this.jsdump('OBJECTION Remove domains:failed', 5);
										}
									}
								}
								else
								{
									this.jsdump('OBJECTION Remove domains:failed:(either no domains entry or known issue in Firefox 2.x)', 5);
								}
							}
						}
					}
				}
			}
		}
	},

	privateBrowsing: function(privateEvent)
	{
		this.getDebugLevel();

		var inPrivateBrowsingMode = false;

		// Check the current state of private browsing mode (If Private Browsing Service is not available then exit)
		try
		{
			var pbs = Components.classes["@mozilla.org/privatebrowsing;1"].getService(Components.interfaces.nsIPrivateBrowsingService);
			inPrivateBrowsingMode = pbs.privateBrowsingEnabled;
		}
		catch (err)
		{
			return null;
		}

		// Check the user preference for using Private Browsing Mode
		var usePrivate = false;
		try
		{
			usePrivate = this.preferences.getBoolPref("usePrivate");
		}
		catch (err) {}

		// Test all the platforms to see which one this is
		var flashAppDirPref = this.testFlashDir();

		// If there is a valid path we can continue
		if (flashAppDirPref !== null)
		{
			// Init the File/Directory objects
			var flashAppDir = new this.nsILocalFile();
			flashAppDir.initWithPath(flashAppDirPref);

			var nsIPrivateMode = Components.classes['@objection/privatemode;1'].getService().wrappedJSObject;
			var privateSuccess = false;

			if (privateEvent === 'pref') // If the user changed the preference then determine what to do
			{
				if (inPrivateBrowsingMode)
				{
					if (usePrivate)
					{
						privateSuccess = nsIPrivateMode.enter(flashAppDir);
					}
					else
					{
						privateSuccess = nsIPrivateMode.exit(flashAppDir);
					}
				}
			}
			else if (usePrivate && privateEvent === 'enter')
			{
				privateSuccess = nsIPrivateMode.enter(flashAppDir);
			}
			else if (usePrivate && privateEvent === 'exit')
			{
				privateSuccess = nsIPrivateMode.exit(flashAppDir);
			}
		}
	},

	testFlashDir: function()
	{
		var tempFlashDirPref = "";
		var tempFlashDir = new this.nsILocalFile();
		var allPlatform = this.prefPlatform.getChildList("", {});
		for (var i=0; i<allPlatform.length; i++)
		{
			if (this.prefPlatform.prefHasUserValue(allPlatform[i]) && typeof(this.prefPlatform.getCharPref(allPlatform[i])) != 'undefined')
			{
				tempFlashDirPref = this.prefPlatform.getCharPref(allPlatform[i]);
				if (tempFlashDirPref !== '')
				{
					try
					{
						tempFlashDir.initWithPath(tempFlashDirPref);
						if (tempFlashDir.exists() && tempFlashDir.isDirectory())
						{
							this.jsdump('OBJECTION Platform:' + allPlatform[i] + ':yes', 4);
							return tempFlashDirPref;
						}
					}
					catch(err)
					{
						this.jsdump('OBJECTION Platform:' + allPlatform[i] + ':no', 1);
					}
				}
			}
		}
		if (this.preferences.prefHasUserValue("flashAppDir") && typeof(this.preferences.getCharPref("flashAppDir")) != 'undefined')
		{
			try
			{
				tempFlashDirPref = this.preferences.getCharPref("flashAppDir");
				tempFlashDir.initWithPath(tempFlashDirPref);
				if (tempFlashDir.exists() && tempFlashDir.isDirectory())
				{
					this.jsdump('OBJECTION Platform:Old:yes', 4);
					return tempFlashDirPref;
				}
			}
			catch(err)
			{
				this.jsdump('OBJECTION Platform:Old:no', 1);
			}
		}
		return null;
	},

	getDebugLevel: function()
	{
		// Get the debug level
		// Levels are: 1-All alerts, 5-Failure alerts, 10-No alerts
		this.debugLevel = 10;
		try {this.debugLevel = this.preferences.getIntPref("debugLevel");}catch (err) {}
	},

	jsdump: function(str, level)
	{
		if (level >= this.debugLevel)
		{
  			Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).logStringMessage(str);
		}
	}
};


/***********************************************************
XPCOM
***********************************************************/

/***********************************************************
constants
***********************************************************/

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// UUID uniquely identifying our component
// You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here
const CLASS_ID = Components.ID("{10060352-006A-11DD-84D9-524756D89593}");

// description
const CLASS_NAME = "Objection Cleanup Javascript XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@objection/cleanup;1";

// observer name
const MY_OBSERVER_NAME = "ObjectionCleanup";

/***********************************************************
class definition
***********************************************************/

//class constructor
function ObjectionCleanup() {
	// If you only need to access your component from Javascript, uncomment the following line:
	//this.wrappedJSObject = this;
};

// class definition
ObjectionCleanup.prototype = {

  // define the functions we want to expose in our interface
	observe: function(aSubject, aTopic, aData)
	{
		switch(aTopic)
		{
			case "xpcom-startup":
				// this is run very early, right after XPCOM is initialized, but before
				// user profile information is applied. Register ourselves as an observer
				// for 'profile-after-change', 'private-browsing' and 'quit-application'.
				var obsSvc = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
				obsSvc.addObserver(objCleanupCall, "profile-after-change", false);
				obsSvc.addObserver(objCleanupCall, "private-browsing", false);
				obsSvc.addObserver(objCleanupCall, "quit-application", false);
				break;

			case "xpcom-shutdown":
				var obsSvc = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
				obsSvc.removeObserver(objCleanupCall, "profile-after-change");
				obsSvc.removeObserver(objCleanupCall, "private-browsing");
				obsSvc.removeObserver(objCleanupCall, "quit-application");
				break;

			default:
				throw Components.Exception("Unknown topic: " + aTopic);
		}
	},

  QueryInterface: function(aIID)
  {
	if (!aIID.equals(Components.interfaces.nsIObserver) &&
		!aIID.equals(nsISupports))
	  throw Components.results.NS_ERROR_NO_INTERFACE;
	return this;
  }
};

/***********************************************************
class factory
***********************************************************/
var ObjectionCleanupFactory = {
  createInstance: function (aOuter, aIID)
  {
		if (aOuter != null)
	  	throw Components.results.NS_ERROR_NO_AGGREGATION;
		return (new ObjectionCleanup()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var ObjectionCleanupModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
	aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
	aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
	
	var catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
	catman.addCategoryEntry("xpcom-startup", MY_OBSERVER_NAME, CONTRACT_ID, true, true);
	catman.addCategoryEntry("xpcom-shutdown", MY_OBSERVER_NAME, CONTRACT_ID, true, true);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
	var catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
	catman.deleteCategoryEntry("xpcom-startup", MY_OBSERVER_NAME, true);
	aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
	aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
	if (!aIID.equals(Components.interfaces.nsIFactory))
	  throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

	if (aCID.equals(CLASS_ID))
	  return ObjectionCleanupFactory;

	throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return ObjectionCleanupModule; }
