/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is by Trevor Hobson
*
* The Initial Developer of the Original Code is
*   Trevor Hobson
* Portions created by the Initial Developer are Copyright (C) 2008
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

var objPrivate = {
// Either enter or exit Objection private mode
// Do not mistake this for the browser Private Browsing Mode

	// Constants for nsIZipXXX
	// See https://developer.mozilla.org/en/nsIZipWriter#Adding_a_comment_to_a_Zip_file
	PR_RDONLY: 0x01,
	PR_WRONLY: 0x02,
	PR_RDWR: 0x04,
	PR_CREATE_FILE: 0x08,
	PR_APPEND: 0x10,
	PR_TRUNCATE: 0x20,
	PR_SYNC: 0x40,
	PR_EXCL: 0x80,

	// Prompt service. For debugging.
	promptService: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),

	nsILocalFile: new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile"),
	preferences: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),

	enter: function(flashAppDir)
	{
		// Check to make sure a directory was passed
		try
		{
			if (!flashAppDir.isDirectory() || !flashAppDir.exists())
			{
				return null;
			}
		} catch(err)
		{
			return null;
		}

		// Read the debug level for this session
		objPrivateGeneric.readDubugLevel();

		// Create a reference to the zip file
		var privateObjectionFile = new this.nsILocalFile();
		privateObjectionFile.initWithFile(flashAppDir);
		privateObjectionFile.append('privateobjection.zip');

		// Add the files to the zip file
		this.zipAddTree(flashAppDir, privateObjectionFile);

	},

	zipAddTree: function(zipTreeRoot, zipFile)
	{
		// Create the zip file
		var zipW = new Components.Constructor("@mozilla.org/zipwriter;1", "nsIZipWriter");
		zipW.open(zipFile, this.PR_RDWR | this.PR_CREATE_FILE | this.PR_TRUNCATE);
		this.zipAddTreeBranch(zipTreeRoot, zipW, '');
		zipW.close();
		objPrivateGeneric.jsdump('objPrivate enter:zip closed:', 1);

	},

	zipAddTreeBranch: function(zipTreeBranch, zipObject, pathRelative)
	{
		// Add the directories and files to the zip file
		try
		{
			var tempPath = '';
			var entries = zipTreeBranch.directoryEntries;
			while(entries.hasMoreElements())
			{
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);
				if (entry.isDirectory())
				{
					if (pathRelative !== '')
					{
						tempPath = pathRelative + '/' + entry.leafName;
					}
					else
					{
						tempPath = entry.leafName;
					}
					zipObject.addEntryDirectory(tempPath, entry.lastModifiedTime, false);
					objPrivateGeneric.jsdump('objPrivate zipAddTreeBranch:add folder:' + entry.path, 2);
					this.zipAddTreeBranch(entry, zipObject, tempPath);
				}
				if (entry.isFile())
				{
					if (entry.leafName !== 'privateobjection.zip') // We could put the zip elsewhere, but cannot guarantee read/write access
					{
						zipObject.addEntryFile(pathRelative + '/' + entry.leafName, Components.interfaces.nsIZipWriter.COMPRESSION_DEFAULT, entry, false);
						objPrivateGeneric.jsdump('objPrivate zipAddTreeBranch:add file:' + entry.path, 2);
					}
				}
			}
		} catch(err)
		{
			objPrivateGeneric.jsdump('objPrivate zipAddTreeBranch:add error:' + entry.path, 5);
		}
	},

	exit: function(flashAppDir)
	{
		// Check to make sure a directory was passed
		try
		{
			if (!flashAppDir.isDirectory() || !flashAppDir.exists())
			{
				return null;
			}
		} catch(err)
		{
			return null;
		}

		// Read the debug level for this session
		objPrivateGeneric.readDubugLevel();

		// Create a reference to the zip file
		var privateObjectionFile = new this.nsILocalFile();
		privateObjectionFile.initWithFile(flashAppDir);
		privateObjectionFile.append('privateobjection.zip');

		// If the zip file does not exist then we cannot restore to pre private mode. The existance of the file determines if we were in Objection Private Mode
		try
		{
			if (!privateObjectionFile.isFile() || !privateObjectionFile.exists())
			{
				return null;
			}
		} catch(err)
		{
			return null;
		}

		// First let us delete all the directories in the Flash Player directory
		try
		{
			var entries = flashAppDir.directoryEntries;
			while(entries.hasMoreElements())
			{
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);
				if (entry.isDirectory())
				{
					try
					{
						entry.remove(true);
						objPrivateGeneric.jsdump('objPrivate exit:Remove directory:success:' + entry.path, 2);
					}
					catch (err)
					{
						objPrivateGeneric.jsdump('objPrivate exit:Remove directory:failed:' + entry.path + ' err:' + err, 5);
					}
				}
			}
		} catch(err) {}

		var zipR = Components.classes["@mozilla.org/libjar/zip-reader;1"].createInstance(Components.interfaces.nsIZipReader);
		zipR.open(privateObjectionFile);
		zipR.test(null);
		// Extract the directory structure from the zip file
		objPrivateGeneric.jsdump('objPrivate exit:extract folders:', 3);
		var entries = zipR.findEntries('*/');
		while (entries.hasMore())
		{
			var zipEntryName = entries.getNext();
			var zipEntryLocal = this.toLocalFile(flashAppDir, zipEntryName);
			objPrivateGeneric.jsdump('objPrivate exit:extract folder:' + zipEntryLocal.path, 3);
			if (!zipEntryLocal.exists())
			{
				try
				{
					zipEntryLocal.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE, 0777);
				}
				catch (err)
				{
					objPrivateGeneric.jsdump('objPrivate exit:extract folders:failed:' + zipEntryLocal.path, 5);
				}
			}
		}
		// Extract the files from the zip file
		objPrivateGeneric.jsdump('objPrivate exit:extract folders:', 3);
		var entries = zipR.findEntries(null);
		while (entries.hasMore())
		{
			var zipEntryName = entries.getNext();
			var zipEntryLocal = this.toLocalFile(flashAppDir, zipEntryName);
			if (zipEntryLocal.exists())
			{
				if (zipEntryLocal.isFile()) // All existing files should have been deleted, just make sure
				{
					zipEntryLocal.remove(true);
				}
				else // It does not matter if a directory already exists
				{
					continue;
				}
			}
			objPrivateGeneric.jsdump('objPrivate exit:extract file:' + zipEntryLocal.path, 3);
			zipR.extract(zipEntryName, zipEntryLocal);
		}
		zipR.close();

		// Delete the zip file
		objPrivateGeneric.jsdump('objPrivate exit:delete zip:', 3);
		privateObjectionFile.remove(true);

		// Set the privateExit preference so the GUI knows we have finished
		var nowDate = new Date();
		this.preferences.setCharPref("privateExit", nowDate.toUTCString());

// To use jsdump effectivly in the exit function you need to uncomment the following line. This is because Private Browsing Mode clears the Error Console when exiting.
//this.promptService.alert(null,'objPrivate','Hold Error Console');
	},
	// Append a relative path (from a zip file) to a nsIFile/LocalFile 
	toLocalFile: function(pathParent, zipEntry)
	{
		var localFile = pathParent.clone();
		var folders = zipEntry.split("/");
		for (var i = 0; i < folders.length; ++i)
		{
			localFile.append(folders[i]);
		}
		return localFile;
	},

};

var objPrivateGeneric = {

	debugLevel: 10,

	readDubugLevel: function()
	{
		this.debugLevel = 10;
		try {this.debugLevel = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.").getIntPref("debugLevel");}catch (err) {}
	},

	jsdump: function(str, level)
	{
		if (level >= this.debugLevel)
		{
  			Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).logStringMessage(str);
		}
	}
};

/***********************************************************
XPCOM
***********************************************************/

/***********************************************************
constants
***********************************************************/

// reference to the interface defined in nsIPrivateMode.idl
const nsIPrivateMode = Components.interfaces.nsIPrivateMode;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// UUID uniquely identifying our component
// You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here
const CLASS_ID = Components.ID("{bf56dd00-2453-11de-8c30-0800200c9a66}");

// description
const CLASS_NAME = "Objection Private Mode Javascript XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@objection/privatemode;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function PrivateMode() {
	// If you only need to access your component from Javascript, uncomment the following line:
	this.wrappedJSObject = this;
};

// class definition
PrivateMode.prototype = {

  // define the functions we want to expose in our interface
  enter: function(flashAppDir) {
      return objPrivate.enter(flashAppDir);
  },

  exit: function(flashAppDir) {
	  return objPrivate.exit(flashAppDir);
  },

  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIPrivateMode) &&
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myPrivateMode = Components.classes['@objection/privatemode;1'].getService().wrappedJSObject;

***********************************************************/
var PrivateModeFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter !== null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new PrivateMode()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var PrivateModeModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return PrivateModeFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return PrivateModeModule; }
